-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("tagQuestLog","AceEvent-3.0", "AceHook-3.0")

-----------------------
--- Library loading ---
-----------------------
local LTQL = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-TagQuestLog")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local min, sub, len = math.min, string.sub, string.len
local GetNumQuestLogEntries, GetQuestLogTitle, HybridScrollFrame_GetOffset = GetNumQuestLogEntries, GetQuestLogTitle, HybridScrollFrame_GetOffset

-----------------
--- variables ---
-----------------
module.modName = LTQL["Quest Log Tag"]

function module:OnInitialize()
	self:SetEnabledState(core:getModuleEnabled("tagQuestLog"))
end

function module:OnEnable()
	self:SecureHook("QuestLog_Update")
	QuestLogScrollFrame:HookScript("OnMouseWheel", function() QuestLog_Update() end)
	QuestLogScrollFrameScrollBar:HookScript("OnValueChanged", function() QuestLog_Update() end)
end

function module:QuestLog_Update()
	local buttons = QuestLogScrollFrame.buttons
	local numButtons = #buttons
	local offset, numEntries, numQuests = HybridScrollFrame_GetOffset(QuestLogScrollFrame), GetNumQuestLogEntries()
	for i=1, numButtons do
		local index = i + offset
		if index > numEntries then break end
		local title, level, qtag, group, header, _, complete, daily, uid = GetQuestLogTitle(index)
		local questLogTitle = buttons[i]
		local questTitleTag, questCheck, questNumGroupMates = questLogTitle.tag, questLogTitle.check, questLogTitle.groupMates
		if questCheck then questCheck:Hide() end
		if not header and uid then
			level, qtag, group, daily, complete = level or 0, qtag or "", group or 0, daily or 0, complete or 0
			if not core.tags[qtag] then
				core:Print("tag ["..qtag.."] is missing for quest ["..uid.."] please report the following error to tomQuest2 author on WOWAce or WOWInterface")
				if level > 0 or group > 0 or daily == 1 then
					questLogTitle:SetText("  ["..(level > 0 and level or "")..(group > 0 and group or "")..(daily == 1 and (qtag ~= DAILY and core.db.profile.quests[core.tags[DAILY]] or "") or "").."]"..title)
				else
					questLogTitle:SetText("  "..title)
				end
			else
				if level > 0 or qtag ~= "" or group > 0 or daily == 1 then
					questLogTitle:SetText("  ["..(level > 0 and level or "")..core.db.profile.quests[core.tags[qtag]]..(group > 0 and group or "")..(daily == 1 and (qtag ~= DAILY and core.db.profile.quests[core.tags[DAILY]] or "") or "").."]"..title)
				else
					questLogTitle:SetText("  "..title)
				end
			end
			if complete == 0 then
				questTitleTag:Hide()
			end
			if IsQuestWatched(index) then
				questCheck:Show()
			end
			local numMates = questNumGroupMates:GetText() or ""
			local numMatesLength = len(numMates)
			if numMatesLength > 0 then
				numMates = sub(numMates, 2, numMatesLength - 1)
				questNumGroupMates:SetText("<"..numMates..">")
				questNumGroupMates:Show()
			else
				questNumGroupMates:Hide()
			end
			QuestLogTitleButton_Resize(questLogTitle)
		end
	end
end